﻿
    Public Sub Main(ByVal Parms As Object)
        ''
        '' Version 0.1
        '' 
        '' This script is designed to read SNMP OID's from Enviromux Mini LXO and put the results in HS devices.
        '' 
        '' The device settings and OID's are stored in an INI file.
        ''
        '' Simplicity is my goal here.  We are doing no writes!
        ''
        '' I use the name target to refer to a system with one or more OIDs to read.  This was less
        '' confusing than calling it a device or system
		''
		'' Most of the functions here were found on the net.  The general setup of this script (and a  
		'' couple of the functions) are copied from the stylings of Jon00 with many thanks to him for 
		'' sharing.
        ''
		'' If I have infringed on anyone, please bring it to my attention and I will correct 
		'' it immediately, as it was not my intention.
		'' 
		'' Please feel free to change this file as you see fit for your purposes.  

        Dim INI_File As String = "Enviromux.ini"
        Dim ScriptName As String = "Enviromux"
        Dim Logging As Integer = CInt(hs.GetINISetting("Global", "ErrorLogging", "", INI_File))
        Dim NumberTargets As Integer = CInt(hs.GetINISetting("Global", "NumberTargets", "", INI_File))
        ' Dim Target(NumberTargets) As Integer
        Dim TargetIP As String
        Dim TargetCommunity As String
        Dim NumberOIDs As Integer
        Dim DevExists As Long
        Dim Target As Integer = 0
        Dim Counter As Integer = 0
        Dim Results As String
        Dim OID() As String
        Dim HSDeviceCode() As String
        Dim HSDeviceName() As String
        Dim ResultType() As String
        Dim HSDivider() As Double
        Dim HSIcon() As String
        Dim TimeResult As String
        Dim iSec As Long
        Dim HTMLString As String




        Try

            ' Logging
            If Logging > 1 Then
                hs.WriteLog(ScriptName, "Target count is " & NumberTargets & ".")
                hs.WriteLog(ScriptName, "Logging set to " & Logging & ".")
            End If

            ' Loop once for each target
            For Target = 1 To NumberTargets
                TargetIP = hs.GetINISetting("Target" & Target, "IPAddress", "", INI_File)
                NumberOIDs = CInt(hs.GetINISetting("Target" & Target, "NumberOIDs", "", INI_File))
                TargetCommunity = hs.GetINISetting("Target" & Target, "Community", "", INI_File)

                ' Logging
                If Logging > 1 Then
                    hs.WriteLog(ScriptName, "Target " & Target & " is located at " & TargetIP & ".")
                    hs.WriteLog(ScriptName, "Target " & Target & " has " & NumberOIDs & " OIDs to read.")
                End If

                ' Validate IP Address
                If IsValidIP(TargetIP) = False Then
                    hs.WriteLog(ScriptName, "Target " & Target & " is an invalid address.")
                    Exit For
                Else
                    If Logging > 1 Then
                        hs.WriteLog(ScriptName, "Target " & Target & " is a valid address.")
                    End If
                End If

                ReDim OID(NumberOIDs)
                ReDim HSDeviceCode(NumberOIDs)
                ReDim HSDeviceName(NumberOIDs)
                ReDim ResultType(NumberOIDs)
                ReDim HSDivider(NumberOIDs)
                ReDim HSIcon(NumberOIDs)

                ' Load OID Arrays
                For Counter = 1 To NumberOIDs
                    OID(Counter) = hs.GetINISetting("Target" & Target, "OID" & Counter, "", INI_File)
                    HSDeviceCode(Counter) = hs.GetINISetting("Target" & Target, "HSDeviceCode" & Counter, "", INI_File)
                    HSDeviceName(Counter) = hs.GetINISetting("Target" & Target, "HSDeviceName" & Counter, "", INI_File)
                    ResultType(Counter) = hs.GetINISetting("Target" & Target, "ResultType" & Counter, "", INI_File)
                    HSDivider(Counter) = CDbl(hs.GetINISetting("Target" & Target, "HSDivider" & Counter, "", INI_File))
                    HSIcon(Counter) = hs.GetINISetting("Target" & Target, "HSIcon" & Counter, "", INI_File)

                    ' Logging
                    If Logging > 1 Then
                        hs.WriteLog(ScriptName, "Target " & Target & ", OID " & Counter & " is " & OID(Counter) & ".")
                        hs.WriteLog(ScriptName, "Target " & Target & ", OID " & Counter & " is device " & HSDeviceCode(Counter) & ".")
                        hs.WriteLog(ScriptName, "Target " & Target & ", OID " & Counter & " is named " & HSDeviceName(Counter) & ".")
                        hs.WriteLog(ScriptName, "Target " & Target & ", OID " & Counter & " uses icon " & HSIcon(Counter) & ".")
                    End If
                Next


                ' Confirm/Create Homeseer Devices
                For Counter = 1 To NumberOIDs
                    DevExists = hs.DeviceExists(HSDeviceCode(Counter))
                    If DevExists = -1 Then ' If it doesn't exist create it
                        CreateDevice(HSDeviceCode(Counter), HSDeviceName(Counter))
                    End If
                Next


                ' Loop once for each OID on this Target
                For Counter = 1 To NumberOIDs
                    Results = SimpleSNMPGet(TargetIP, TargetCommunity, OID(Counter))
                    If Logging > 1 Then
                        hs.WriteLog(ScriptName, "Target " & Target & ", OID " & Counter & " returned " & Results & ".")
                    End If

                    Select Case ResultType(Counter)
                        Case "Text"
                            If HSIcon(Counter) = "" Then
                                hs.SetDeviceString(HSDeviceCode(Counter), Results, True)
                            Else
                                HTMLString = "<table border='0' cellpadding='0' cellspacing='0' width='100%'>"
                                HTMLString &= "<tr><td align='left' width='1'><img src='" & HSIcon(Counter) & "'></td>"
                                HTMLString &= "<td align='left' class='tablecellcustom'>" & Results & "</td></tr></table>"
                                hs.SetDeviceString(HSDeviceCode(Counter), HTMLString, True)
                                hs.SetDeviceLastChange(HSDeviceCode(Counter), Now())
                            End If
                        Case "Numeric"
                            If HSIcon(Counter) = "" Then
                            Else
                                HTMLString = "<table border='0' cellpadding='0' cellspacing='0' width='100%'>"
                                HTMLString &= "<tr><td align='left' width='1'><img src='" & HSIcon(Counter) & "'></td>"
                                If HSDivider(Counter) = "10" Then
                                  HTMLString &= "<td align='left' class='tablecellcustom'>" & Val(Results) / HSDivider(Counter) & "°C</td></tr></table>"
                                Else
                                  HTMLString &= "<td align='left' class='tablecellcustom'>" & Val(Results) / HSDivider(Counter) & "%</td></tr></table>"
                                End If
                                hs.SetDeviceString(HSDeviceCode(Counter), HTMLString, True)
                                hs.SetDeviceLastChange(HSDeviceCode(Counter), Now())
                            End If

                            hs.SetDeviceValue(HSDeviceCode(Counter), Val(Results) / HSDivider(Counter))
                        Case "Both"
                            If HSIcon(Counter) = "" Then
                                hs.SetDeviceString(HSDeviceCode(Counter), Results, True)
                            Else
                                HTMLString = "<table border='0' cellpadding='0' cellspacing='0' width='100%'>"
                                HTMLString &= "<tr><td align='left' width='1'><img src='" & HSIcon(Counter) & "'></td>"
                                HTMLString &= "<td align='left' class='tablecellcustom'>" & Results & "</td></tr></table>"
                                hs.SetDeviceString(HSDeviceCode(Counter), HTMLString, True)
                                hs.SetDeviceLastChange(HSDeviceCode(Counter), Now())
                            End If
                            hs.SetDeviceValue(HSDeviceCode(Counter), Val(Results) / HSDivider(Counter))
                        Case "Time"
                            iSec = Val(Results) / HSDivider(Counter)
                            hs.SetDeviceValue(HSDeviceCode(Counter), CInt(iSec))
                            TimeResult = Format(iSec \ 3600, "00") & "Hours " & Format((iSec Mod 3600) \ 60, "00") & _
                                        "Minutes " & Format(iSec Mod 60, "00") & "Seconds"

                            If HSIcon(Counter) = "" Then
                                hs.SetDeviceString(HSDeviceCode(Counter), TimeResult, True)
                            Else
                                HTMLString = "<table border='0' cellpadding='0' cellspacing='0' width='100%'>"
                                HTMLString &= "<tr><td align='left' width='1'><img src='" & HSIcon(Counter) & "'></td>"
                                HTMLString &= "<td align='left' class='tablecellcustom'>" & TimeResult & "</td></tr></table>"
                                hs.SetDeviceString(HSDeviceCode(Counter), HTMLString, True)
                                hs.SetDeviceLastChange(HSDeviceCode(Counter), Now())
                            End If

                    End Select
                Next

            Next


        Catch ex As Exception
            hs.WriteLog(ScriptName, "Error (" & ex.Message & ")")
        End Try


    End Sub
    Function CreateDevice(ByVal UDevcode As String, ByVal HSDeviceName As String) As Object


        Dim ScriptName As String = "Enviromux"
        Dim DevRef As Object
        Dim DV As Object


        Try
            DevRef = hs.NewDeviceRef(HSDeviceName)
            DV = hs.GetDeviceByRef(DevRef)
            DV.location = "Enviroment"
            DV.hc = Left(UDevcode, 1)
            DV.dc = Mid(UDevcode, 2)
            DV.misc = &H10
            DV.dev_type_string = "Enviromux"
            DV = Nothing
            DevRef = Nothing
            hs.WriteLog(ScriptName, "Enviromux enviroment device created for Devicecode " & UDevcode & ".")

        Catch ex As Exception
            hs.WriteLog(ScriptName, "Error C: " & ex.Message)

        End Try

        Return (0)

    End Function
    Public Function SimpleSNMPGet(ByVal ip As String, ByVal community As String, ByVal oid As String) As String

        Dim ScriptName As String = "Enviromux"
        Dim snmp1 As String
        Dim snmp2 As String
        Dim receiver As New System.Net.IPEndPoint(System.Net.IPAddress.Parse(ip), 161)
        Dim snmp_response As Lextm.SharpSnmpLib.Variable


        Try
            Dim oidvar As New Lextm.SharpSnmpLib.Variable(Lextm.SharpSnmpLib.ObjectIdentifier.Convert(oid))
            Dim varList As New System.Collections.Generic.List(Of Lextm.SharpSnmpLib.Variable)()
            varList.Add(oidvar)
            snmp_response = Lextm.SharpSnmpLib.Messaging.Messenger.Get(Lextm.SharpSnmpLib.VersionCode.V2, receiver, New Lextm.SharpSnmpLib.OctetString(community), varList, 10000)(0)


            snmp1 = snmp_response.Id.ToString().Trim()
            snmp2 = snmp_response.Data.ToString().Trim()
            If snmp2 = "NoSuchInstance" Then
                Return (Nothing)
            Else
                Return (snmp2)
            End If
        Catch e As Exception
            hs.WriteLog(ScriptName, "ERROR---SIMPLE-GET--->>>")
            hs.WriteLog(ScriptName, "Source: " & e.Source.ToString())
            hs.WriteLog(ScriptName, "Message: " & e.Message.ToString())
            Return (Nothing)
        End Try
    End Function

    '<summary>
    'method to validate an IP address using
    'the TryParse Method of the IPAddress class
    '</summary>
    '<param name="addr">address to validate</param>
    '<returns></returns>
    Public Function IsValidIP(ByVal addr As String) As Boolean
        'boolean variable to hold the status
        Dim valid As Boolean = False
        'check to make sure an ip address was provided
        If String.IsNullOrEmpty(addr) Then
            'address wasnt provided so return false
            valid = False
        Else
            'use TryParse to see if this is a 
            'valid ip address. TryParse returns a
            'boolean based on the validity of the
            'provided address, so assign that value
            'to our boolean variable
            valid = Net.IPAddress.TryParse(addr, Nothing)
        End If
        'return the value
        Return valid
    End Function
